//
//  HWDownloadManager.h
//  VclustersGemini
//
//  Created by 余衡武 on 2020/4/24.
//  Copyright © 2020 APPLE. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SGDownloadManager.h"
#import "SGDownloadSession.h"
#import "SGDownloadQueue.h"
#import "SGDownloadOperation.h"
#import "SGCacheManager.h"
#import "NSString+SGHashString.h"
#import "NSURLSession+SGDownloadTask.h"
#import "HWDownloadModel.h"

NS_ASSUME_NONNULL_BEGIN

/** 下载中通知 */
#define  SGDownloadTaskExeing        @"SGDownloadTaskExeing"
/** 下载失败通知 */
#define  SGDownloadTaskExeError      @"SGDownloadTaskExeError"
/** 下载完成通知 */
#define  SGDownloadTaskExeEnd        @"SGDownloadTaskExeEnd"
/** 下载暂停/取消通知 */
#define  SGDownloadTaskExeSuspend    @"SGDownloadTaskExeSuspend"
/** 删除下载任务通知 */
#define  SGDownloadTaskExeDelete     @"SGDownloadTaskExeDelete"
/** 移除loading通知 */
#define  SGDownloadRemoveLoading     @"SGDownloadRemoveLoading"

@interface HWDownloadManager : NSObject


/** 操作记录plist文件保存路径 */
@property (nonatomic,copy)NSString *plistPath;
/** 登录账号信息字典 */
@property (nonatomic,copy)NSDictionary *LoginInfo;
/** 总任务数 */
@property (nonatomic,strong)NSMutableArray *allTasks;
/** 正在下载中的任务 */
@property (nonatomic,strong)NSMutableArray *downloadingTasks;
/** 等待下载的任务 */
@property (nonatomic,strong)NSMutableArray *downloadWaitTasks;
/** 最大下载任务数 */
@property (nonatomic, assign) int downloadMaxNum;



//获得管理类单例对象
+ (instancetype)shardInstance;

/** 创建下载任务 */
- (void)creatDownloadTaskWithModel:(HWDownloadModel *)model;

/** 开启所有下载任务 */
- (void)startAllDownloadTask;
/** 暂停所有下载任务 */
- (void)pauseAllDownloadTask;
/** 清除所有下载任务 */
- (void)clearAllDownloadTask;
/** 清除选中的下载任务 */
- (void)deleteSelectedDownloadTask;
/** 清除指定的下载任务 */
- (void)deleteDownloadTaskWithModel:(HWDownloadModel *)model;

// 新建某一个下载任务
- (void)startDownloadTaskWithModel:(HWDownloadModel *)model;
// 开始某一个下载任务
- (void)taskResumeWithModel:(HWDownloadModel *)model;
// 暂停某一个下载任务
- (void)pauseDownloadTaskWithModel:(HWDownloadModel *)model;
// 恢复某一个下载任务
- (void)resumeDownloadTaskWithModel:(HWDownloadModel *)model;

@end

NS_ASSUME_NONNULL_END
