//
//  SGDownloadOperation.h
//  OfflineBreakPointDownload
//
//  Created by Shangen Zhang on 16/11/26.
//  Copyright © 2016年 Shangen Zhang. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SGCacheManager.h"


SG_EXTERN NSString * const SGDownloadCompleteNoti;

typedef void(^SGReceiveResponseOperation)(NSString *filePath);
typedef void(^SGReceivDataOperation)(NSInteger completeSize,NSInteger expectSize);
typedef void(^SGCompleteOperation)(NSDictionary *respose,NSError *error);

/** 下载状态*/
typedef  enum : NSUInteger {
    DownloadStateWaiting = 0,   /** 下载等待中 */
    DownloadStateDoing,         /** 下载中 */
    DownloadStateSuspended,     /** 下载暂停 */
    DownloadStateCompleted,     /** 下载完成 */
    DownloadStateFailed,        /** 下载失败 */
} DownloadStateType;

@protocol SGDownloadOperationProtocol <NSObject>
                                    
// 供queue管理方法

// 处理响应值
- (void)operateWithResponse:(NSURLResponse *)response;
// 处理接收到的碎片
- (void)operateWithReceivingData:(NSData *)data;
// 处理完成回调
- (void)operateWithComplete:(NSError *)error;


/**
 设置block回调

 @param didReceiveResponse 开始下载的回调
 @param didReceivData 接收到下载的回调
 @param didComplete 下载完成的回调
 */
- (void)configCallBacksWithDidReceiveResponse:(SGReceiveResponseOperation)didReceiveResponse
                                didReceivData:(SGReceivDataOperation)didReceivData
                                  didComplete:(SGCompleteOperation)didComplete;

@end

@interface SGDownloadOperation : NSObject <SGDownloadOperationProtocol>

/** 绑定的标示及task的创建 */
@property (readonly,nonatomic, copy)NSString *url;

/** 下载任务 */
@property (nonatomic,strong)NSURLSessionDataTask *dataTask;

/** 文件句柄 可以记录文件的下载的位置 */
@property (nonatomic,strong) NSFileHandle *handle;

/** 下载的文件总大小 */
@property (nonatomic,assign) int64_t totalSize;

/** 当前下载了多少 */
@property (nonatomic,assign) int64_t currentSize;

/** 当前下载文件名称 */
@property (nonatomic,copy) NSString *fileName;

/** 当前下载文件沙盒全路径 */
@property (nonatomic,copy) NSString *fullPath;

/** 文件下载状态 */
@property (nonatomic,assign) DownloadStateType downloadState;

/** 下载完成-时间戳 */
@property (nonatomic,assign)  NSInteger timeStamp;

/** 文件类型 */
@property (nonatomic,assign)  NSInteger fileType;

/** 是否完成 */
@property (nonatomic, assign)  NSInteger isFinished;



// 创建下载操作任务
- (instancetype)initWith:(NSString *)url session:(NSURLSession *)session;

- (instancetype)initWith:(NSString *)url session:(NSURLSession *)session fileType:(NSInteger)fileType fileSize:(NSInteger)fileSize;



- (NSDictionary *)downLoadInfoWithFinished:(BOOL)finished;

@end

