//
//  SGDownloadQueue.m
//  OfflineBreakPointDownload
//
//  Created by Shangen Zhang on 16/11/26.
//  Copyright © 2016年 Shangen Zhang. All rights reserved.
//

#import "SGDownloadQueue.h"
#import "SGDownloadOperation.h"
#import "SGDownloadManager.h"
#import "NSURLSession+SGDownloadTask.h"
#import <MJExtension.h>

@interface SGDownloadQueue ()
// 列队管理集合
@property (nonatomic,strong) NSMutableArray <SGDownloadOperation *> *operations;

@end

@implementation SGDownloadQueue

- (instancetype)init {

    if (self = [super init]) {
        _maxCount = 3;
        //_maxCount = 5;
        [self registeNotification];
    }
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)didResiveDownloadFileCompete:(NSNotification *)noti {
    SGDownloadOperation *operation = noti.object;
    if (operation) {
        [self.operations removeObject:operation];
    }
}

- (NSMutableArray *)getOperationDoing {
    
    NSMutableArray *doingArray = [NSMutableArray array];
    for (SGDownloadOperation *operation in self.operations) {
        if (operation.downloadState == DownloadStateDoing) {
            [doingArray addObject:operation];
        }
    }
    return doingArray;
}

- (NSMutableArray *)getOperationWaiting {
    
    NSMutableArray *waitingArray = [NSMutableArray array];
    [self.operations enumerateObjectsUsingBlock:^(SGDownloadOperation * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        
        SGDownloadOperation *operation = (SGDownloadOperation *)obj;
        if (operation.downloadState == DownloadStateWaiting) {
            [waitingArray addObject:operation];
        }
    }];
    return waitingArray;
}

#pragma mark - handle Out operations
- (void)addDownloadWithSession:(NSURLSession *)session URL:(NSURL *)url begin:(void(^)(NSString *))begin progress:(void(^)(NSInteger,NSInteger))progress complete:(void(^)(NSDictionary *,NSError *))complet {
    // 获取operation对象
    SGDownloadOperation *operation = [self operationWithUrl:url.absoluteString];
    
    if (operation == nil) { // 之前不存在此任务
        
        operation = [[SGDownloadOperation alloc] initWith:url.absoluteString session:session];
        
        if (operation == nil) {
            // 没有下载任务代表已下载完成
            NSDictionary *fileInfo = [SGCacheManager queryFileInfoWithUrl:url.absoluteString];
            if (fileInfo && complet) {
                complet(fileInfo,nil);
            }else {
                complet(nil,[NSError errorWithDomain:@"构建下载任务失败" code:-1 userInfo:nil]);
            }
            return;
        }
        
        [self.operations addObject:operation];
    }
    
    // 回调赋值operation
//    [operation configCallBacksWithDidReceiveResponse:begin didReceivData:progress didComplete:complet];
    
//    [operation.dataTask resume];
    
}

- (void)addDownloadWithSession:(NSURLSession *)session URL:(NSURL *)url fileType:(NSInteger)fileType fileSize:(NSInteger)fileSize begin:(void(^)(NSString * filePath))begin progress:(void(^)(NSInteger completeSize,NSInteger expectSize))progress complete:(void(^)(NSDictionary *respose,NSError *error))complet {
    // 获取operation对象
    SGDownloadOperation *operation = [self operationWithUrl:url.absoluteString];
    
    if (operation == nil) { // 队列里不存在此任务
        
        operation = [[SGDownloadOperation alloc] initWith:url.absoluteString session:session fileType:fileType fileSize:fileSize];
        
        if (operation == nil) {
            // 没有下载任务代表已下载完成
            NSDictionary *fileInfo = [SGCacheManager queryFileInfoWithUrl:url.absoluteString];
            if (fileInfo && complet) {
                complet(fileInfo,nil);
            }else {
                complet(nil,[NSError errorWithDomain:@"构建下载任务失败" code:-1 userInfo:nil]);
            }
            return;
        }
        
        [self.operations addObject:operation];
    }
    
    // 回调赋值operation
//    [operation configCallBacksWithDidReceiveResponse:begin didReceivData:progress didComplete:complet];
    
    if ([self getOperationDoing].count < self.maxCount) { // 下载中任务数少于最大任务限制
        [operation.dataTask resume];
        [self operationStartWithOperation:operation];
    }else { // 下载中任务数大于最大任务限制
//        [operation.dataTask suspend];
        [self operationWaitingWithOperation:operation];
//        HLog(@"下载中任务数等于最大任务限制：%zd",[self getOperationDoing].count);
    }
}

- (void)operateDownloadWithUrl:(NSString *)url session:(NSURLSession *)session handle:(DownloadHandleType)handle {
    // 1、任务列表里取任务
    SGDownloadOperation *operation = [self operationWithUrl:url];
    
    // 2、本地plist文件里提取的任务
    if (!operation) {
        NSDictionary *dict = [SGCacheManager queryFileInfoWithUrl:url];
        operation = [SGDownloadOperation mj_objectWithKeyValues:dict];
    }

    // 3、本地plist文件里提取的任务不存在dataTask
    if (!operation.dataTask && operation.currentSize != operation.totalSize) {
        operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
        [self.operations addObject:operation];
    }
    
    if (operation) {
        
        switch (handle) {
            case DownloadHandleTypeStart:
                
                switch (operation.dataTask.state) {
                    case NSURLSessionTaskStateRunning:
                        HLog(@"NSURLSessionTaskStateRunning");
                        break;
                    
                    case NSURLSessionTaskStateSuspended:
                        HLog(@"NSURLSessionTaskStateSuspended");
                        operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
                        break;
                        
                    case NSURLSessionTaskStateCanceling:
                        HLog(@"NSURLSessionTaskStateCanceling");
                        operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
                        break;
                        
                    case NSURLSessionTaskStateCompleted:
                        HLog(@"NSURLSessionTaskStateCompleted");
                        break;
                        
                    default:
                        break;
                }
                
                if ([self getOperationDoing].count >= self.maxCount) { // 下载中任务数超过最大任务限制
                    HLog(@"下载中的任务数超过最大限制")
//                    [operation.dataTask suspend]; // 暂停
                    [self operationWaitingWithOperation:operation];
                    return;
                }
                
                [operation.dataTask resume]; // 开始
                [self operationStartWithOperation:operation];
                break;
            case DownloadHandleTypeSuspend:
                [operation.dataTask suspend]; // 暂停
                [self operationSuspendWithOperation:operation];
                break;
            case DownloadHandleTypeCancel:
                if (operation.dataTask) { // 任务列表删除任务
                    [operation.dataTask cancel];  // 取消
                    [self.operations removeObject:operation];
                }
                [SGCacheManager deleteFileWithUrl:url];   // plist删除任务
                [self operationDeleteWithOperation:operation];
                break;
        }
    }
}

- (void)cancelAllTasks {
    // 取消所有的任务
    [self.operations enumerateObjectsUsingBlock:^(SGDownloadOperation * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        [obj.dataTask cancel];
    }];
    // 清理内存
    self.operations = nil;
}

- (void)suspendAllTasksWithSession:(NSURLSession *)session {
    // 暂停所有的任务
    [self.operations enumerateObjectsUsingBlock:^(SGDownloadOperation * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        
        SGDownloadOperation *operation = (SGDownloadOperation *)obj;
        if (operation.currentSize != operation.totalSize) { // 暂停未完成的下载任务
            if (!operation.dataTask) { // 给plist里的任务添加dataTask 添加到operations
                operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
            }
            [operation.dataTask suspend];
            [self operationSuspendWithOperation:operation];
        }else {
            HLog(@"已完成的任务");
        }

    }];
}

- (void)startAllTasksWithSession:(NSURLSession *)session {
    
    // 开始所有的任务
    [self.operations enumerateObjectsUsingBlock:^(SGDownloadOperation * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        
        SGDownloadOperation *operation = (SGDownloadOperation *)obj;
        if (!operation.dataTask) { // 给plist里的任务添加dataTask 添加到operations
            operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
        }
  
        if ([self getOperationDoing].count < self.maxCount) { // 下载中任务数少于最大任务限制
            
            switch (operation.dataTask.state) {
                case NSURLSessionTaskStateRunning:
                    HLog(@"NSURLSessionTaskStateRunning");
                    break;
                
                case NSURLSessionTaskStateSuspended:
                    HLog(@"NSURLSessionTaskStateSuspended");
                    break;
                    
                case NSURLSessionTaskStateCanceling:
                    HLog(@"NSURLSessionTaskStateCanceling");
                    operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
                    break;
                    
                case NSURLSessionTaskStateCompleted:
                    HLog(@"NSURLSessionTaskStateCompleted");
                    break;
                    
                default:
                    break;
            }
            
            [operation.dataTask resume]; // 开始
            [self operationStartWithOperation:operation];
        }else { // 下载中任务数大于最大任务限制
//            [operation.dataTask suspend];
            [self operationWaitingWithOperation:operation];
    //        HLog(@"下载中任务数等于最大任务限制：%zd",[self getOperationDoing].count);
        }

    }];
}

#pragma mark-监听通知
- (void)registeNotification {
    // 监听完成通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didResiveDownloadFileCompete:) name:SGDownloadCompleteNoti object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(taskExeEnd:) name:SGDownloadTaskExeEnd object:nil];
//    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(taskExeEnd:) name:SGDownloadTaskExeError object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(taskExeEnd:) name:SGDownloadTaskExeSuspend object:nil];
//    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(taskExeEnd:) name:SGDownloadTaskExeDelete object:nil];
}
    
- (void)taskExeEnd:(NSNotification *)notification
{
    NSMutableArray *operationWaiting = [self getOperationWaiting];
    
    [operationWaiting enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        SGDownloadOperation *operation = obj;
        if ([self getOperationDoing].count < self.maxCount) {
            [self operateDownloadWithUrl:operation.url session:self.session handle:DownloadHandleTypeStart];
            
        }else {
            *stop = YES;
        }
    }];
}

#pragma mark 发送通知
// 暂停某一个operation 保存本地 通知外界
- (void)operationSuspendWithOperation:(SGDownloadOperation *)operation {
    operation.downloadState = DownloadStateSuspended;
    [SGCacheManager saveFileInfoWithDict:[operation downLoadInfoWithFinished:NO]];
    [[NSNotificationCenter defaultCenter] postNotificationName:SGDownloadTaskExeSuspend object:nil userInfo:@{@"operation" : operation}];
}

// 重启某一个operation 保存本地 通知外界
- (void)operationStartWithOperation:(SGDownloadOperation *)operation {
    operation.downloadState = DownloadStateDoing;
    [SGCacheManager saveFileInfoWithDict:[operation downLoadInfoWithFinished:NO]];
    [[NSNotificationCenter defaultCenter] postNotificationName:SGDownloadTaskExeing object:nil userInfo:@{@"operation" : operation}];
}

// 等待某一个operation 保存本地 通知外界
- (void)operationWaitingWithOperation:(SGDownloadOperation *)operation {
    operation.downloadState = DownloadStateWaiting;
    [SGCacheManager saveFileInfoWithDict:[operation downLoadInfoWithFinished:NO]];
    [[NSNotificationCenter defaultCenter] postNotificationName:SGDownloadTaskExeSuspend object:nil userInfo:@{@"operation" : operation}];
}

// 删除某一个operation 保存本地 通知外界
- (void)operationDeleteWithOperation:(SGDownloadOperation *)operation {
    [[NSNotificationCenter defaultCenter] postNotificationName:SGDownloadTaskExeDelete object:nil userInfo:nil];
}

#pragma mark - handle download
- (void)dataTask:(NSURLSessionDataTask *)dataTask didReceiveResponse:(NSURLResponse *)response {
    [[self oprationWithDataTask:dataTask] operateWithResponse:response];
}

- (void)dataTask:(NSURLSessionDataTask *)dataTask didReceiveData:(NSData *)data {
    [[self oprationWithDataTask:dataTask] operateWithReceivingData:data];
}


- (void)task:(NSURLSessionTask *)task didCompleteWithError:(NSError *)error {
    [[self oprationWithDataTask:task] operateWithComplete:error];
}


#pragma mark - query operation
- (SGDownloadOperation *)operationWithUrl:(NSString *)url{
    __block SGDownloadOperation *operation = nil;
    
    [self.operations enumerateObjectsUsingBlock:^(SGDownloadOperation * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if ([obj.url isEqualToString:url]) {
            operation = obj;
            *stop = YES;
        }
    }];
    
    return operation;
}

// 寻找operation
- (SGDownloadOperation *)oprationWithDataTask:(NSURLSessionTask *)dataTask {
    __block SGDownloadOperation *operation = nil;
    
    [self.operations enumerateObjectsUsingBlock:^(SGDownloadOperation * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if (obj.dataTask == dataTask) {
            operation = obj;
            *stop = YES;
        }
    }];
    
    return operation;
}

#pragma mark - lazy load 
- (NSMutableArray<SGDownloadOperation *> *)operations {
    
    if (!_operations) {
        _operations = [NSMutableArray array];
        [self addOperationsFromPlist];
    }
    return _operations;
}

- (void)addOperationsFromPlist {
    
    [self.operations removeAllObjects];
    
    NSMutableArray *operations = [[SGDownloadManager shareManager] getAllOperation];
    for (SGDownloadOperation *operationPlist in operations) {
        
        // 1、任务列表里取任务
        SGDownloadOperation *operation = [self operationWithUrl:operationPlist.url];
        
        // 2、本地plist文件里提取的任务
        if (!operation) {
            NSDictionary *dict = [SGCacheManager queryFileInfoWithUrl:operationPlist.url];
            operation = [SGDownloadOperation mj_objectWithKeyValues:dict];
        }

        // 3、本地plist文件里提取的任务不存在dataTask
        if (!operation.dataTask) {
            operation.dataTask = [self.session sg_downloadDataTaskWithURLString:operation.url startSize:operation.currentSize];
            operation.handle = [NSFileHandle fileHandleForWritingAtPath:operation.fullPath];
//            [operation configCallBacksWithDidReceiveResponse:begin didReceivData:progress didComplete:complet];
            [self.operations addObject:operation];
        }
    }
}

@end
